@@ -28,6 +28,7 @@ import json
 import json
 import textwrap
 import datetime
+import time
 
 import psycopg
 import psycopg.rows
@@ -296,6 +297,64 @@ class XlprDb(Connection):
         print(kw)
         self.insert_row('xlserver', kw)
 
+    def run_api(self, func, buffer, options=None, *,
+                mode_name=None,
+                use_exists_record=True,
+                update_record=True,
+                save_buffer_threshold_size=4 * 1024 ** 2):
+        """ 配合database数据库的情况下，调用API功能
+
+        :param func: 被封装的带执行的api函数
+        :param buffer: 图片数据
+        :param options: 执行api功能的配套采纳数
+        :param mode_name: 可以指定存入数据库的功能名，默认用func的名称
+        :param use_exists_record: 如果数据库里已有记录，是否直接复用
+        :param update_record: 新结果是否记录到数据库
+        :param save_buffer_threshold_size: buffer小余多少，才存储进数据库
+
+        """
+
+        # 1 预处理，参数标准化
+        if options is None:
+            options = {}
+        options = {k: options[k] for k in sorted(options.keys())}  # 对参数进行排序，方便去重
+
+        # 2 调百度识别接口
+        if use_exists_record or update_record:  # 有开数据库，并且复用和更新至少有项开启了
+            if mode_name is None:
+                mode_name = func.__name__
+            # 如果数据库里有处理过的记录，直接引用
+            im_etag = get_etag(buffer)
+            if use_exists_record:
+                res = self.get_xlapi_record(mode=mode_name, etag=im_etag, options=options)
+            else:
+                res = None
+
+            # 否则调用百度的接口识别
+            # TODO 这里使用协程逻辑最合理但配置麻烦，需要func底层等做协程的适配支持
+            #   使用多线程测试了并没有更快，也发现主要耗时是post，数据库不会花太多时间，就先不改动了
+            #   等以后数据库大了，看运行是否会慢，可以再测试是否有必要弄协程
+            if res is None or 'error_code' in res:
+                tt = time.time()
+                res = func(buffer, options)
+                elapse_ms = round_int(1000 * (time.time() - tt))
+
+                if len(buffer) < save_buffer_threshold_size:
+                    self.insert_row2files(buffer, etag=im_etag, name='.jpg')
+                if update_record:
+                    input = {'mode': mode_name, 'etag': im_etag}
+                    if options:
+                        input['options'] = options
+                    xlapi_id = self.insert_row2xlapi(input, res, elapse_ms, on_conflict='REPLACE')
+                    res['xlapi_id'] = xlapi_id
+
+            if 'log_id' in res:  # 有xlapi_id的标记，就不用百度原本的log_id了
+                del res['log_id']
+        else:
+            res = func(buffer, options)
+
+        return res
+
     def __2_host_trace相关可视化(self):
         """ TODO dbview 改名 host_trace """
         pass
